@@ -4,21 +4,21 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '1.14';
+our $VERSION = '1.19';
 
 my %jamo2prim = (
-    '1100', 0x376F,   '1101', 0x3770,   '1102', 0x3771,   '1103', 0x3772,
-    '1105', 0x3774,   '1106', 0x3775,   '1107', 0x3776,   '1109', 0x3778,
-    '110A', 0x3779,   '110B', 0x377A,   '110C', 0x377B,   '110E', 0x377D,
-    '110F', 0x377E,   '1110', 0x377F,   '1111', 0x3780,   '1112', 0x3781,
-    '1161', 0x37ED,   '1162', 0x37EE,   '1163', 0x37EF,   '1165', 0x37F1,
-    '1166', 0x37F2,   '1167', 0x37F3,   '1168', 0x37F4,   '1169', 0x37F5,
-    '116A', 0x37F6,   '116B', 0x37F7,   '116C', 0x37F8,   '116D', 0x37F9,
-    '116E', 0x37FA,   '116F', 0x37FB,   '1170', 0x37FC,   '1171', 0x37FD,
-    '1172', 0x37FE,   '1173', 0x37FF,   '1174', 0x3800,   '1175', 0x3801,
-    '11A8', 0x384B,   '11AB', 0x384E,   '11AF', 0x3852,   '11B7', 0x385A,
-    '11B8', 0x385B,   '11BC', 0x385F,   '11BD', 0x3860,
-); # for DUCET v7.0.0
+    '1100', 0x3A5E,   '1101', 0x3A5F,   '1102', 0x3A60,   '1103', 0x3A61,
+    '1105', 0x3A63,   '1106', 0x3A64,   '1107', 0x3A65,   '1109', 0x3A67,
+    '110A', 0x3A68,   '110B', 0x3A69,   '110C', 0x3A6A,   '110E', 0x3A6C,
+    '110F', 0x3A6D,   '1110', 0x3A6E,   '1111', 0x3A6F,   '1112', 0x3A70,
+    '1161', 0x3ADC,   '1162', 0x3ADD,   '1163', 0x3ADE,   '1165', 0x3AE0,
+    '1166', 0x3AE1,   '1167', 0x3AE2,   '1168', 0x3AE3,   '1169', 0x3AE4,
+    '116A', 0x3AE5,   '116B', 0x3AE6,   '116C', 0x3AE7,   '116D', 0x3AE8,
+    '116E', 0x3AE9,   '116F', 0x3AEA,   '1170', 0x3AEB,   '1171', 0x3AEC,
+    '1172', 0x3AED,   '1173', 0x3AEE,   '1174', 0x3AEF,   '1175', 0x3AF0,
+    '11A8', 0x3B3A,   '11AB', 0x3B3D,   '11AF', 0x3B41,   '11B7', 0x3B49,
+    '11B8', 0x3B4A,   '11BC', 0x3B4E,   '11BD', 0x3B4F,
+); # for DUCET v8.0.0
 
 my(%u2e, $prim, $wt);
 while (<DATA>) {
